CREATE OR REPLACE FUNCTION update_listen_totals()
RETURNS void AS $$
BEGIN
  WITH artist_plays AS (
    SELECT artist_name, COUNT(*)::integer AS total_plays
    FROM listens
    GROUP BY artist_name
  )
  UPDATE artists
  SET total_plays = COALESCE(ap.total_plays, 0)
  FROM artist_plays ap
  WHERE artists.name_string = ap.artist_name;
  WITH album_plays AS (
    SELECT album_key, artist_name, COUNT(*)::integer AS total_plays
    FROM listens
    GROUP BY album_key, artist_name
  )
  UPDATE albums
  SET total_plays = COALESCE(ap.total_plays, 0)
  FROM album_plays ap
  WHERE albums.key = ap.album_key
    AND albums.artist_name = ap.artist_name;
  WITH genre_plays AS (
    SELECT g.id, COUNT(*)::integer AS total_plays
    FROM listens l
    JOIN artists a ON l.artist_name = a.name_string
    JOIN genres g ON a.genres::text = g.id::text
    GROUP BY g.id
  )
  UPDATE genres
  SET total_plays = COALESCE(gp.total_plays, 0)
  FROM genre_plays gp
  WHERE genres.id = gp.id;
END;
$$ LANGUAGE plpgsql;
