CREATE OR REPLACE VIEW optimized_tagged_content AS
SELECT
  unnest(p.tags) AS tag,
  p.title::TEXT,
  p.url::TEXT,
  p.date::timestamp WITHOUT TIME ZONE AS content_date,
  NULL AS author,
  NULL::TEXT AS rating,
  p.featured,
  p.tags::TEXT[],
  'article'::TEXT AS type,
  'Post'::TEXT AS label
FROM optimized_posts p
UNION ALL
SELECT
  unnest(l.tags) AS tag,
  l.title::TEXT,
  l.link::TEXT AS url,
  l.date::timestamp WITHOUT TIME ZONE AS content_date,
  l.author,
  NULL::TEXT AS rating,
  NULL::BOOLEAN AS featured,
  l.tags::TEXT[],
  'link'::TEXT AS type,
  'Link'::TEXT AS label
FROM optimized_links l
UNION ALL
SELECT
  unnest(b.tags) AS tag,
  b.title::TEXT,
  b.url::TEXT,
  b.date_finished::timestamp WITHOUT TIME ZONE AS content_date,
  NULL AS author,
  b.rating::TEXT,
  NULL::BOOLEAN AS featured,
  b.tags::TEXT[],
  'books'::TEXT AS type,
  'Book'::TEXT AS label
FROM optimized_books b
WHERE LOWER(b.status) = 'finished'
UNION ALL
SELECT
  unnest(m.tags) AS tag,
  m.title::TEXT,
  m.url::TEXT,
  m.last_watched::timestamp WITHOUT TIME ZONE AS content_date,
  NULL AS author,
  m.rating::TEXT,
  NULL::BOOLEAN AS featured,
  m.tags::TEXT[],
  'movies'::TEXT AS type,
  'Movie'::TEXT AS label
FROM optimized_movies m
WHERE m.last_watched IS NOT NULL
UNION ALL
SELECT
  unnest(s.tags) AS tag,
  s.title::TEXT,
  s.url::TEXT,
  s.last_watched_at::timestamp WITHOUT TIME ZONE AS content_date,
  NULL AS author,
  NULL::TEXT AS rating,
  NULL::BOOLEAN AS featured,
  s.tags::TEXT[],
  'tv'::TEXT AS type,
  'Show'::TEXT AS label
FROM optimized_shows s
WHERE s.last_watched_at IS NOT NULL;
